#
#	File:		icebreaker.make
#
#	Contains:	Test instrument class (makefile)
#
#	Written by:	pab
#
#	Copyright:	 1994 by The 3DO Company. All rights reserved.
#				This material constitutes confidential and proprietary
#				information of the 3DO Company and shall not be used by
#				any Person or for any purpose except as expressly
#				authorized in writing by the 3DO Company.
#
#	Change History (most recent first):
#
#		<2>	 	 1/24/94	CDE		Upgraded to Jumpstart2-style.
#		<1>		10/28/93	pab		Derived from Makefile.
#

#####################################
#		Symbol definitions
#####################################
Application		=	icebreaker
ObjectDir		=	:Objects:
CPLUS			=	armCPlus
CC				=	armcc
ASM				=	armasm
LINK			=	armlink
StreamDir		=  	Godiva:3DO:Streaming:2p0:DataStream:
SubscriberDir	=  {StreamDir}Subscribers:

DebugFlag		=	1
#DebugFlag		=	0

#####################################
#	Default options
#####################################

CPlusOptions	= -i "{3DOIncludes}" -i "{StreamDir}" -i "{SubscriberDir}" -d __cplusplus -d DEBUG={DebugFlag}

#CPlusOptions	= -i "{3DOIncludes}" -d DEBUG={DebugFlag}

CDebugOptions	= -g
#CDebugOptions	=
COptions		= {CDebugOptions} -fa -za1 -zps0 -i "{3DOIncludes}" -d DEBUG={DebugFlag}

SDebugOptions	= -g
#SDebugOptions	=
SOptions		= {SDebugOptions} -bi -i "{3DOIncludes}"

LDebugOptions	= -d
#LDebugOptions	=
LOptions		= {LDebugOptions} -aif -r -b 0x00

#####################################
#		Object files
#####################################

#NOTE: Add any other required library files here:
LIBS			=	
					"{SubscriberDir}subscriber.lib"		
					"{StreamDir}dataacq.lib"			
					"{StreamDir}ds.lib"					
					"{3DOLibs}codec.lib" 				
					"{3DOLibs}Lib3DO.lib"				
					"{3DOLibs}CPlusLib.lib"				
					"{3DOLibs}operamath.lib"			
					"{3DOLibs}filesystem.lib"			
					"{3DOLibs}graphics.lib"				
					"{3DOLibs}audio.lib"				
					"{3DOLibs}music.lib"				
					"{3DOLibs}input.lib"				
					"{3DOLibs}clib.lib"					
					"{3DOLibs}swi.lib"


#NOTE: Add any other required object files here:
OBJECTS			=	"{ObjectDir}landscape.cp.o" 	
					"{ObjectDir}sounds.cp.o" 		
					"{ObjectDir}dudemeyer.cp.o" 	
					"{ObjectDir}seeker.cp.o" 		
					"{ObjectDir}animation.cp.o" 	
					"{ObjectDir}solids.cp.o"	 	
					"{ObjectDir}weapon.cp.o"	 	
					"{ObjectDir}deadlist.cp.o"	 	
					"{ObjectDir}levels.cp.o"	 	
					"{ObjectDir}nvram.cp.o"	 		
					"{ObjectDir}userif.cp.o"	 	
					"{ObjectDir}FontHandler.cp.o"	
					"{ObjectDir}memcheck.cp.o"		
					"{ObjectDir}PlayMusic.cp.o"		
					"{ObjectDir}PrepareStream.cp.o"	
					"{ObjectDir}{Application}.cp.o"


#####################################
#	Default build rules
#####################################
All					{Application}

{ObjectDir}			:

.cp.o				.cp
	{CPLUS} {CPlusOptions} {DepDir}{Default}.cp -o {TargDir}{Default}.cp.o -comp {CC} {COptions}

.c.o				.c
	{CC} {COptions} -o {TargDir}{Default}.c.o {DepDir}{Default}.c

.s.o				.s
	{ASM} {SOptions} -o {TargDir}{Default}.s.o {DepDir}{Default}.s


#####################################
#	Target build rules
#####################################
{Application}			{Application}.make {LIBS} {OBJECTS}
	{LINK}	{LOptions}					
			-o {Application}			
			"{3DOLibs}cstartup.o"		
			{OBJECTS}					
			{LIBS}
	SetFile {Application} -c 'EaDJ' -t 'PROJ'
	modbin {Application} -stack 6000
#	modbin {Application} -stack 4000 -debug
	stripaif "{Application}" -o "{Application}"
#	duplicate "{Application}" "{Application}".sym "{3DORemote}" {3DOAutodup}
	duplicate "{Application}" "{3DORemote}" {3DOAutodup}

#####################################
#	Include file dependencies
#####################################
{ObjectDir}{Application}.cp.o			"{3DOLibs}"Lib3DO.lib {Application}.make landscape.h    
																				 animation.h    
																				 sounds.h		
																				 seeker.h		
																				 deadlist.h		
																				 dudemeyer.h	
																				 levels.h		
																				 nvram.h		
																				 userif.h		
																				 solids.h		
																				 FontHandler.h	
																				 weapon.h		
																				 PlayMusic.h	
																				 memcheck.h		
																				 icebreaker.h
{ObjectDir}sounds.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make sounds.h
{ObjectDir}landscape.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make landscape.h    
																				 animation.h    
																				 icebreaker.h
{ObjectDir}solids.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make solids.h		
																				 animation.h    
																				 landscape.h	
																				 seeker.h		
																				 icebreaker.h
{ObjectDir}animation.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make animation.h	
																				 solids.h		
																				 icebreaker.h
{ObjectDir}seeker.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make seeker.h   	
																				 deadlist.h		
																				 sounds.h		
																				 landscape.h	
																				 dudemeyer.h	
																				 solids.h		
																				 weapon.h		
																				 animation.h    
																				 icebreaker.h
{ObjectDir}dudemeyer.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make dudemeyer.h   	
																				 solids.h		
																				 deadlist.h		
																				 sounds.h		
																				 icebreaker.h
{ObjectDir}weapon.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make weapon.h   	
																				 animation.h    
																				 seeker.h		
																				 landscape.h	
																				 deadlist.h		
																				 solids.h		
																				 sounds.h		
																				 icebreaker.h
{ObjectDir}deadlist.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make deadlist.h   	
																				 animation.h    
																				 solids.h		
																				 icebreaker.h
{ObjectDir}levels.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make icebreaker.h  	
																				 levels.h		
																				 landscape.h    
																				 animation.h    
																				 PlayMusic.h	
																				 solids.h		
																				 seeker.h
{ObjectDir}nvram.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make icebreaker.h  	
																				 levels.h		
																				 userif.h		
																				 PlayMusic.h	
																				 nvram.h
{ObjectDir}userif.cp.o					"{3DOLibs}"Lib3DO.lib {Application}.make icebreaker.h  	
																				 levels.h		
																				 sounds.h		
																				 seeker.h		
																				 PlayMusic.h	
																				 animation.h	
																				 FontHandler.h	
																				 solids.h		
																				 memcheck.h		
																				 userif.h		
																				 nvram.h
{ObjectDir}PrepareStream.cp.o			"{3DOLibs}"Lib3DO.lib {Application}.make PrepareStream.h
{ObjectDir}PlayMusic.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make icebreaker.h   
																				 levels.h		
																				 PlayMusic.h    
																				 PrepareStream.h
{ObjectDir}FontHandler.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make FontHandler.h
{ObjectDir}memcheck.cp.o				"{3DOLibs}"Lib3DO.lib {Application}.make memcheck.h
